#!/bin/bash
#
#       musicdb_add_dir.sh
#       
#       Copyright © 2011 Brandon Invergo <brandon@brandoninvergo.com>
#
#       This file is part of Coredesktop.
#       
#       Coredesktop is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Coredesktop is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Coredesktop.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################################
#                                                                             #
# This is an example script of how to build a simple music library from a     #
# (nested) directory of audio files, in conjunction with the                  #
# musicdb_store_song.awk AWK script.                                          #
#                                                                             #
# TODO: can this be adapted to work in a Makefile to handle updating the      #
#       library when new files have been added?                               #
#                                                                             #
# TODO: 'extract' is the main bottleneck in performance and should be         #
#       replaced by something designed better for scripting                   #
#                                                                             #
###############################################################################

scripts_dir=~/Projects/coredesktop/src/examples/musicdb
db_dir=~/.local/share/coredesktop/db/music

function try_id3 {
    for version in id3v2 id3v23 id3v24 id3; do
        data=`extract -n -l$version "$1"`
        if ((`echo "$data" | wc | awk '{ print $1 }'` != 1)); then
            echo "$data" | awk -v db_dir="$db_dir" -v filename="$PWD/$1" -f $scripts_dir/musicdb_store_song.awk
            return;
        fi;
    done
}

function add_song {
    echo "$1"
    filetype=`echo "$1" | cut -f2 -d.` 
    case "$filetype" in
        mp3) try_id3 "$1" ;;
        ogg) extract -n -logg "$1" | \
            awk -v db_dir="$db_dir" -v filename="$PWD/$1" -f $scripts_dir/musicdb_store_song.awk ;;
        flac) extract -n -lflac "$1" | \
            awk -v db_dir="$db_dir" -v filename="$PWD/$1" -f $scripts_dir/musicdb_store_song.awk ;;
    esac
}

function process_file {
    if `test -d "$1"`; then
        pushd "$1";
        for filename in *; do
            process_file "$filename";
        done
        popd;
    else add_song "$1";
    fi;
}
process_file "$1"
