#!/bin/bash
#
#       musicdb_search.sh
#       
#       Copyright © 2011 Brandon Invergo <brandon@brandoninvergo.com>
#
#       This file is part of Coredesktop.
#       
#       Coredesktop is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Coredesktop is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Coredesktop.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################################
#                                                                             #
# This script gives an example of how to search the music database. It        #
# operates on some assumptions for how we normally search for music in a      #
# collection. A music collection is arranged hierarchically such that an      #
# artist has one or more albums, and each album has one or more songs.        #
# Typically, we'll want to retrieve a list of all the albums by an artist,    #
# or a list of all the songs on an album. These can be combined to retrieve   #
# all the songs by an artist. But typically we probably won't be searching    #
# for all the artists who have written a song. This sort of hierarchical      #
# search is good for filtering a library. Sometimes, though, a free search    #
# for a keyword is desirable.                                                 #
#                                                                             #
# Two types of searches are implemented here. If you search for an artist or  #
# an album, a list of all the artist's albums or the album's songs will be    #
# printed, respecively. Searching for an album requires that the artist name  #
# be provided. The output may then be used to query the databases directly.   #
# If you search for a song, you can use any search term to query the song     #
# table (which includes artist and album info). The former  search method     #
# takes advantage of the roughly O(1) search time offered by the DBM database #
# format. The latter uses grep, which will be a bit slower.                   #
#                                                                             #
###############################################################################

db_dir=~/.local/share/coredesktop/db/music

optname=""
artist=""
album=""
song=""
until [[ $optname == "?" ]]; do
    getopts "a:b:s:" optname
    case $optname in
        a) artist=$OPTARG ;;
        b) album=$OPTARG ;;
        s) song=$OPTARG ;;
    esac
done

if [[ $song != "" ]]; then
    dbf -a $db_dir/song.db | grep "$song";
elif [[ $artist != "" && $album == "" ]]; then
    echo "$artist" | dbf $db_dir/artist.db;
elif [[ $artist != "" && $album != "" ]]; then
    echo "$artist~$album" | dbf $db_dir/album.db;
fi
