#!/usr/bin/awk
#
#       musicdb_store_song.awk
#       
#       Copyright © 2011 Brandon Invergo <brandon@brandoninvergo.com>
#
#       This file is part of Coredesktop.
#       
#       Coredesktop is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Coredesktop is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Coredesktop.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################################
#                                                                             #
# This awk script takes the output from GNU extract (part of the libextractor #
# package), formats it in a more-easily parsible way than the extract's '-g'  #
# argument does, with fields defined for music files (which could easily be   #
# changed to use any of the ~100 metadata fields supported by extract). This  #
# works on the fairly safe assumption that the id3 tags won't contain the \t  #
# delimiter.                                                                  #
#                                                                             #
###############################################################################

BEGIN{
    # This is the delimiter separating fields from values:
    FS = " - "
    }
{
    if(NR > 1){
        # The first line says "Keywords for file <filename>:"
        # so skip it. 
        fields[$1] = $2
    }
}
END{
    # Now that we have everything, add it all to the database.
    if (fields["artist"] == ""){ fields["artist"] = "Unknown" }
    if (fields["album"] == ""){ fields["album"] = "Unknown" }
    if (fields["title"] == ""){ fields["title"] = "Unknown" }
    if (fields["track number"] == ""){ fields["track number"] = "Unknown" }
    if (fields["performer"] == ""){ fields["performer"] = fields["artist"] }
    fields["album artist"] = fields["performer"]
    
    artist_search = ("echo \"" fields["album artist"] "\" | dbf " db_dir "/artist.db")

    album_key = fields["album artist"] "~" fields["album"]
    album_search = ("echo \"" album_key "\" | dbf " db_dir "/album.db")

    song_key = album_key "~" fields["title"]
    song_info = fields["title"] "\t" filename "\t" fields["artist"] "\t" fields["album"] "\t" fields["track number"] 

    artist_search | getline artist_res
    if (artist_res == ""){
        add_artist = ("echo \"" fields["album artist"] "|" album_key "\" | dbs " db_dir "/artist.db")
        add_artist | getline nothing
        close(add_artist)
    }
    else{
        if(index(artist_res, album_key) == 0){
            new_artist_info = artist_res "\t" album_key
            update_artist = ("echo \"" fields["album artist"] "|" new_artist_info "\" | dbs " db_dir "/artist.db")   
            update_artist | getline nothing
            close(update_artist)
        }
    }
    close(artist_search)

    album_search | getline album_res
    if (album_res == ""){
        add_album = ("echo \"" album_key "|" song_key "\" | dbs " db_dir "/album.db")
        add_album | getline nothing
        close(add_album)
    }
    else{
        if(index(album_res, song_key) == 0){
            new_album_info = album_res "\t" song_key
            update_album = ("echo \"" album_key "|" new_album_info "\" | dbs " db_dir "/album.db")   
            update_album | getline nothing
            close(update_album)
        }
    }
    close(album_search)

    add_song = ("echo \"" song_key "|" song_info "\" | dbs " db_dir "/song.db")
    add_song | getline nothing
}
